# Introduction

Flutter routing 2.0 requires a lot of setup. The goal of this package is to make that setup easy by providing a simpler API.

Flutter brings web navigation and app navigation together and with that resulted some confusion. The confusion lies in the difference between the back button on the web and the up button in an app. The API will also make it clear that those are not to be confused for one and another.

See the material navigation principles for more information on the back and up buttons: https://developer.android.com/guide/navigation/navigation-principles

# Wording

During the design phase of this document `up stack` is used to describe the back stack to clearly delimitate the difference between up and back buttons.


# Design goals

  - The pages on the stack have to be evident
  - Seemless experience between web and mobile (up and back buttons acting the same)
  - Reactive resolvers, navigation based on state (eg: auth state)
  - Passing parameters easily
  - Allow users to specify the transitions they want if needed
  - state can be known easily

## Scenario

For designing the API this fairly common and complex scenario will be taken as the base case to solve:

```
  '/loading' => user access this page when the authentication state is unknown
  '/sign-in' => user access this page when unauthenticated
  '/sign-in/phone' => user can access this nested page when unauthenticated 
  '/create-profile' => when user is authenticated but doesn't have a profile 
  '/dashboard' => when the user is authenticated & has a profile, the dashboard
  '/products' => when the user is authenticated & has a profile and clicked on a menu item to see the products
  '/products/:id' => when the user wants to see a specific product
```